/**
 * 
 */
package exlabel.structure;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.QuadCurve2D;
import java.awt.geom.Rectangle2D;

import exlabel.util.ExRenderer;
import exlabel.util.TextStroke;

/**
 * @author fireball
 */
public class ExEdge implements IExEdge 
{

	ExNode startNode;

	ExNode endNode;

	ExRenderer render;

	String label;

	Shape shape;
	
	GeneralPath lodshape;
	
	float curvature;

	Double cx, cy;

	public ExEdge() 
	{

		this.startNode = new ExNode();
		this.endNode = new ExNode();

	}

	public ExEdge(ExNode startNode, ExNode endNode, String label,
			ExRenderer render) 
	{

		this.startNode = startNode;
		this.endNode = endNode;
		this.render = render;

		this.label = label;
		
		this.curvature = curvature;
		this.lodshape = new GeneralPath();	

		update();

	}

	public void draw(Graphics2D g2d, int mode) 
	{
		if (mode > 0) {
			update();
		}

		if (ExGraph.DRAW_STANDARD_EDGES) 
		{
			render.drawEdge(this, g2d, mode);
		} else 
		{
			render.drawLabel(this, g2d, mode);
		}

	}

	public void update() 
	{

		
		//CREATING FULL SHAPE
	//	System.out.println("now updating...");

	//	System.out.print("now updating... |\t");

		Float x1 = (float) this.getStartNode().getXpos();
		Float y1 = (float) this.getStartNode().getYpos();
		Float x2 = (float) this.getEndNode().getXpos();
		Float y2 = (float) this.getEndNode().getYpos();
		Float w = (x2 - x1);
		Float h = (y2 - y1);

		Double l = Math.abs(Math.sqrt((w * w + h * h)));

		/*
		 * cx = Math.cos(Math.atan(y1/x1)+alpha); cy =
		 * Math.sin(Math.atan(y1/x1)+alpha);
		 */

		Float arc = (float) (ExGraph.STANDARD_LABEL_ARC * (l)) / 500;
		cx = x1 + w / 2 + arc * -h / l;
		cy = y1 + h / 2 + arc * w / l;

		if (ExGraph.STANDARD_LABEL_FONT_SIZE > 10 && ExGraph.DRAW_PERFORMANCE_MODE == false) 
		{
			String s = this.label;
			Font f = new Font(ExGraph.STANDARD_LABEL_FONT, Font.PLAIN,
					ExGraph.STANDARD_LABEL_FONT_SIZE);

			shape = new QuadCurve2D.Float(x1, y1, cx.intValue(), cy.intValue(),	x2, y2);
			//System.out.println("offset: " + startNode.getRadius() + "  x: "+ x1 + "  y: " + y1);

			TextStroke Ts = new TextStroke(s, f, startNode.getRadius()/2 + ExGraph.STANDARD_NODE_EDGE_LABEL_OFFSET);
			shape = Ts.createStrokedShape(shape);
		}
		
		//CREATING LOD SHAPE
		float arc2;
		
		
		//TEMP ARC HEIGHT
		if (ExGraph.STANDARD_LABEL_FONT_SIZE < 10) {
			arc2 = ExGraph.STANDARD_LABEL_FONT_SIZE / 1.6f;
		} else  {
			arc2 = 10/1.6f;
		}
		
				
		this.lodshape = new GeneralPath();
		
		Double qx1 = x1 + arc2* -h/ l;
		Double qy1 = y1 - arc2*  -w/ l;		
		Double qx2 = x1 - arc2* -h/ l;
		Double qy2 = y1 + arc2*  -w/ l;
		
		Double cx = x1 + w/2 + arc* -h/ l;
		Double cy = y1 + h/2 + arc* w/ l;		
		
		lodshape.moveTo( x2, y2 );
		lodshape.quadTo( cx.intValue(), cy.intValue(), qx1.intValue(), qy1.intValue());
		lodshape.lineTo(qx2.intValue(), qy2.intValue());
		lodshape.quadTo( cx.intValue(), cy.intValue(),  x2, y2 );
		

	}

	public boolean intersects(double x, double y)
	{
		if (!ExGraph.DRAW_STANDARD_EDGES) {
			if (lodshape.contains(x, y))
			{
				return true;
			} else
				return false;
		} else {
			
			//TODO: implement standard Arrow intersection
			return false;
		}
	}

	public ExNode getEndNode()
	{
		return endNode;
	}

	public void setEndNode(ExNode endNode) 
	{
		this.endNode = endNode;
	}

	public String getLabel() 
	{
		return label;
	}

	public void setLabel(String label) 
	{
		label.trim();
		this.label = label;
	}

	public ExNode getStartNode() 
	{
		return startNode;
	}

	public void setStartNode(ExNode startNode) 
	{
		this.startNode = startNode;
	}

	public Shape getShape() 
	{
		return shape;
	}

	public void setShape(Shape shape) 
	{
		this.shape = shape;
	}

	public Double getCx() 
	{
		return cx;
	}

	public void setCx(Double cx) 
	{
		this.cx = cx;
	}

	public Double getCy() 
	{
		return cy;
	}

	public void setCy(Double cy) 
	{
		this.cy = cy;
	}

	
	public GeneralPath getLodshape()
	{
		return lodshape;
	}

	
	public void setLodshape(GeneralPath lodshape)
	{
		this.lodshape = lodshape;
	}

	
	public float getCurvature()
	{
		return curvature;
	}

	
	public void setCurvature(float curvature)
	{
		this.curvature = curvature;
	}

	/**
	 * @param args
	 */

}
