package exlabel.structure;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Iterator;

import exlabel.util.ExRenderer;

public class ExGraph implements IExGraph
{

	public static int STANDARD_NODE_RADIUS = 20;

	public static int STANDARD_NODE_EDGE_LABEL_OFFSET = 2;

	public static int STANDARD_MOUSEOVER_RADIUS_ADD = 4;

	public static float STANDARD_NODE_RESIZE_BIAS = 0.5f;

	public static float STANDARD_LABEL_ALPHA = 1.0f;

	public static String STANDARD_LABEL_FONT = "Helvetica";

	public static int STANDARD_LABEL_FONT_SIZE = 25;

	public static int STANDARD_LABEL_ARC = 100;

	public static int STANDARD_EDGE_SIZE = 2;

	public static int STANDARD_TEMP_EDGE_SIZE = 1;

	public static float STANDARD_NODE_MASS = 1;

	public static Color STANDARD_NODE_COLOR = Color.red;

	public static Color STANDARD_ACTIVE_NODE_COLOR = Color.black;

	public static Color STANDARD_TEMP_NODE_COLOR = Color.gray;

	public static Color STANDARD_MOUSEOVER_NODE_COLOR = Color.orange;

	public static Color STANDARD_EDGE_COLOR = Color.blue;

	public static Color STANDARD_MOUSEOVER_EDGE_COLOR = Color.orange;

	public static Color STANDARD_ACTIVE_EDGE_COLOR = Color.orange;

	public static Color STANDARD_TEMP_EDGE_COLOR = Color.red;

	public static Color STANDARD_LABEL_COLOR = Color.black;

	public static Color STANDARD_TEMP_LABEL_COLOR = Color.darkGray;

	public static Color STANDARD_MOUSEOVER_LABEL_COLOR = Color.orange;

	public static String STANDARD_EDGE_LABEL = "Information Visualization 2005.";

	public static String STANDARD_NODE_LABEL = "Node Label";

	public static boolean DRAW_STANDARD_EDGES = false;

	public static boolean DRAW_NODE_LABELS = false;

	public static boolean DRAW_PERFORMANCE_MODE = false;

	private ArrayList NodeList;

	private ArrayList EdgeList;

	private ArrayList AdjacencyList;

	private ExRenderer render;

	private ExNode insertDraggingTempNode;

	private ExEdge insertDraggingTempEdge;

	private ExNode mouseOverTempNode;

	private ExEdge mouseOverTempEdge;

	private ExNode activeNode;

	private ExEdge activeEdge;

	private boolean drawTempNode;

	private boolean drawTempEdge;

	private int graphSizeX, graphSizeY;

	private String filename;

	public ExGraph(String filename, int x, int y)
	{

		this.filename = filename;
		this.graphSizeX = x;
		this.graphSizeY = y;
		this.NodeList = new ArrayList();
		this.EdgeList = new ArrayList();
		this.AdjacencyList = new ArrayList();

		this.drawTempNode = false;
		this.drawTempEdge = false;

		this.render = new ExRenderer();
		// render.start();
		this.insertDraggingTempNode = new ExNode(0, 0, 0, STANDARD_NODE_RADIUS,
				STANDARD_NODE_MASS, STANDARD_NODE_LABEL, render);
		this.insertDraggingTempEdge = new ExEdge(new ExNode(), new ExNode(),
				STANDARD_EDGE_LABEL, render);
		// readFile(filename);

		this.mouseOverTempNode = null;
		this.mouseOverTempEdge = null;

	}

	public void draw(Graphics2D g2d)
	{

		// active Node drawing
		if (activeNode != null)
		{

			activeNode.draw(g2d, 3);

		}

		// Drawing all edges
		Iterator ita = EdgeList.iterator();
		while (ita.hasNext())
		{

			ExEdge temp = (ExEdge) ita.next();
			temp.draw(g2d, 0);
		}

		// active Edge drawing
		if (activeEdge != null)
		{

			activeEdge.draw(g2d, 3);

		}

		// Drawing Mouse Over Edge

		if (mouseOverTempEdge != null)
		{

			mouseOverTempEdge.draw(g2d, 2);

		}
		// drawing user feedback edge
		if (drawTempEdge)
		{
			insertDraggingTempEdge.draw(g2d, 1);
		}

		// drawing Mouse Over Node
		if (mouseOverTempNode != null)
		{
			mouseOverTempNode.draw(g2d, 2);// draw MouseoverTemp node

		}

		// Drawing all nodes
		Iterator itb = NodeList.iterator();

		while (itb.hasNext())
		{

			ExNode temp = (ExNode) itb.next();
			temp.draw(g2d, 0);
		}

		// drawing user feedback node
		if (drawTempNode)
		{
			insertDraggingTempNode.draw(g2d, 1);
		}

	}

	public void update()
	{

		Iterator it = NodeList.iterator();
		while (it.hasNext())
		{

			ExNode temp = (ExNode) it.next();
			temp.update();
		}

		Iterator it2 = EdgeList.iterator();
		while (it2.hasNext())
		{

			ExEdge temp = (ExEdge) it2.next();
			temp.update();
		}

	}

	public void setSize(int x, int y)
	{

		this.graphSizeX = x;
		this.graphSizeY = y;
	}

	public ExNode hitNode(int x, int y)
	{

		Iterator it = NodeList.iterator();
		while (it.hasNext())
		{

			ExNode temp = (ExNode) it.next();
			if ((x > temp.getXpos() - temp.getRadius())
					&& (x < temp.getXpos() + temp.getRadius())
					&& (y > temp.getYpos() - temp.getRadius())
					&& (y < temp.getYpos() + temp.getRadius()))
			{

				return temp;

			}
		}

		return null;

	}

	public int hitNodeInt(int x, int y)
	{

		Iterator it = NodeList.iterator();
		while (it.hasNext())
		{

			ExNode temp = (ExNode) it.next();
			if ((x > temp.getXpos() - temp.getRadius())
					&& (x < temp.getXpos() + temp.getRadius())
					&& (y > temp.getYpos() - temp.getRadius())
					&& (y < temp.getYpos() + temp.getRadius()))
			{

				System.out.println(NodeList.indexOf(temp));
				return NodeList.indexOf(temp);

			}
		}

		return -1;

	}

	public ExEdge hitEdge(int x, int y)
	{

		Iterator it = EdgeList.iterator();
		while (it.hasNext())
		{
			ExEdge temp = (ExEdge) it.next();
			if (temp.intersects((double) x, (double) y))
			{
				return temp;
			}
		}

		return null;

	}

	public ExNode getNodeById(int i)
	{

		if (i >= 0)
		{
			return (ExNode) NodeList.get(i);
		} else
			return null;

	}

	public boolean addNode(ExNode newnode)
	{

		// insert node into node list
		if (render != null)
		{
			if (NodeList.add(newnode))
			{
				ArrayList newList = new ArrayList();
				newList.add(newnode);
				AdjacencyList.add(newList);
				return true;
			} else
				return false;
		} else
		{
			// DEBUG
			System.out.println("Node not set, Renderer not added.");
			return false;
		}

		// insert node into Adjacency list

	}

	public boolean addEdge(ExEdge newedge)
	{

		if (render != null)
		{
			if (EdgeList.add(newedge))
			{
				if (getAdjacenzByStartNode(newedge.getStartNode()) != null)
				{

					getAdjacenzByStartNode(newedge.getStartNode()).add(
							newedge.getEndNode());
					triggerNodeRadiusCalc(getAdjacenzByStartNode(newedge
							.getStartNode()));
					System.out.println("added edge to adjacency list");
					return true;
				}
				return true;
			} else
				return false;
		} else
		{
			// DEBUG
			System.out.println("Edge not set, Renderer not added.");
			return false;
		}
	}

	public boolean setInsertDraggingTempNode(int xpos, int ypos, int zpos,
			int radius, float mass)
	{
		insertDraggingTempNode.setPosition(xpos, ypos, zpos);
		insertDraggingTempNode.setRadius(radius);
		insertDraggingTempNode.setMass(mass);
		return true;
	}

	public boolean setInsertDraggingTempEdge(ExNode startNode, ExNode endNode,
			String Label)
	{

		insertDraggingTempEdge.setStartNode(startNode);
		insertDraggingTempEdge.setEndNode(endNode);
		insertDraggingTempEdge.setLabel(Label);
		return true;
	}

	private void triggerNodeRadiusCalc(ArrayList ex)
	{

		ArrayList tempNode = (ArrayList) AdjacencyList.get(AdjacencyList
				.indexOf(ex));
		int massVal = tempNode.size();

		// Radius Calculation
		ExNode startNode = (ExNode) tempNode.get(0);

		int tempRadius = startNode.getRadius() + massVal
				+ (int) STANDARD_NODE_RESIZE_BIAS;

		if (tempRadius < 40)
		{
			startNode.setRadius(tempRadius);
		}

		this.update();

	}

	private void triggerNodeCurvatureCalc(ArrayList ex)
	{

		int counterArray[] = new int[ex.size()];
		Iterator it = ex.iterator();
		it.next();
		while (it.hasNext())
		{

		}

	}

	private ArrayList getAdjacenzByStartNode(ExNode start)
	{

		Iterator it = AdjacencyList.iterator();
		while (it.hasNext())
		{
			ArrayList templist = (ArrayList) it.next();
			if (templist.get(0).equals(start))
			{

				return (ArrayList) templist;

			}
		}
		return null;

	}

	public ExRenderer getRender()
	{
		return render;
	}

	public void disableTempNodeDrawing()
	{

		drawTempNode = false;

	}

	public void enableTempNodeDrawing()
	{

		drawTempNode = true;

	}

	public void disableTempEdgeDrawing()
	{

		drawTempEdge = false;

	}

	public void enableTempEdgeDrawing()
	{

		drawTempEdge = true;

	}

	public ExNode getMouseOverTempNode()
	{
		return mouseOverTempNode;
	}

	public void setMouseOverTempNode(ExNode mouseOverTempNode)
	{
		this.mouseOverTempNode = mouseOverTempNode;
	}

	public ExEdge getMouseOverTempEdge()
	{
		return mouseOverTempEdge;
	}

	public void setMouseOverTempEdge(ExEdge mouseOverTempEdge)
	{
		this.mouseOverTempEdge = mouseOverTempEdge;
	}

	public ExEdge getActiveEdge()
	{
		return activeEdge;
	}

	public void setActiveEdge(ExEdge activeEdge)
	{
		this.activeEdge = activeEdge;
	}

	public ExNode getActiveNode()
	{
		return activeNode;
	}

	public void setActiveNode(ExNode activeNode)
	{
		this.activeNode = activeNode;
	}

}
