package exlabel.util;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import exlabel.structure.ExEdge;
import exlabel.structure.ExGraph;
import exlabel.structure.ExNode;

public class ExRenderer implements IExRenderer
{

	public boolean drawNode(ExNode node, Graphics2D g2d, int mode)
	{

		AlphaComposite myAlpha;
		int tempRadius;

		switch (mode)
		{

		case 0: // NORMAL NODE DRAWING
		{

			// /// STANDARD NODE
			myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1);
			g2d.setComposite(myAlpha);

			g2d.setColor(ExGraph.STANDARD_NODE_COLOR);
			g2d.fillOval(node.getXpos() - node.getRadius() / 2, node.getYpos()
					- node.getRadius() / 2, node.getRadius(), node.getRadius());

			// ///////standardlabel

			if (ExGraph.DRAW_NODE_LABELS)
			{

				FontRenderContext frc = g2d.getFontRenderContext();
				Font f = new Font(ExGraph.STANDARD_LABEL_FONT, Font.PLAIN,
						(int) (ExGraph.STANDARD_LABEL_FONT_SIZE/ 1.6f));

				g2d.setColor(ExGraph.STANDARD_LABEL_COLOR);

				String s = node.getLabel();
				TextLayout tl = new TextLayout(s, f, frc);
				if (ExGraph.DRAW_STANDARD_EDGES)
				{
					tl.draw(g2d, (float) (node.getXpos() + node.getRadius()),
							(float) node.getYpos());
				} else if (!ExGraph.DRAW_PERFORMANCE_MODE)
				{

					GlyphVector gv = f.createGlyphVector(frc, s);
					GlyphVector gv2 = f.createGlyphVector(frc, ".");
					int length = gv.getNumGlyphs();
					float radius = node.getRadius() + 20;

					double charAngle = 0;
					float startangle = 45;

					float fullAngle = 0;
					
					fullAngle = (float) ((s.length() -1) * ExGraph.STANDARD_LABEL_FONT_SIZE * 1.25
					/ (radius / 20));
					
					startangle = (fullAngle/2);
					for (int i = 0; i < length; i++)
					{
						Point2D p = gv.getGlyphPosition(i);
						AffineTransform at = new AffineTransform();
						at.setToTranslation(node.getXpos(), node.getYpos()); // rot
																				// um
																				// centerx
						charAngle = ExGraph.STANDARD_LABEL_FONT_SIZE * 1.25 * i
								/ (radius / 20);

						
						if (charAngle > 300)
							break;

						at.rotate(-Math.toRadians(startangle - charAngle));// ++
																			// suppa
						at.translate(-(p.getX() + gv.getGlyphMetrics(i)
								.getBounds2D().getCenterX()), -p.getY()
								- radius / 2); // gedreht um centerx

						// System.out.println("char: "+s.charAt(i)+" angle:
						// "+charAngle+" \t x:
						// "+gv.getGlyphMetrics(i).getBounds2D().getX()+"\t
						// width:
						// "+gv.getGlyphMetrics(i).getBounds2D().getWidth()+"\t
						// advance: "+gv.getGlyphMetrics(i).getAdvance());

						Shape glyph = gv.getGlyphOutline(i);
						Shape transformedGlyph = at
								.createTransformedShape(glyph);
						g2d.fill(transformedGlyph);
					}
				}

			}

			// /label
			// Float Angle = (float) (Math.atan2(node.getRadius(), 0.0D) * 2D);
			/*
			 * AffineTransform at = g2d.getTransform(); GlyphVector gv =
			 * getFont().createGlyphVector(new FontRenderContext(at, true,
			 * true), s); for (int i = 0; i < s.length(); i++) { Rectangle2D
			 * letterRect = gv.getGlyphOutline(i).getBounds2D(); double w =
			 * letterRect.getWidth(); double a =
			 * gv.getGlyphMetrics(i).getAdvance(); if (w < 0.0D) { w = a; }
			 * CharWidth[i] = w; CharDists[i] = TextDist + w / 2D; TextDist += i !=
			 * s.length() - 1 ? a : w; }
			 */
			break;
		}
		case 1: // TEMP NODE DRAWING
		{
			// setting alpha
			myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
			g2d.setComposite(myAlpha);

			g2d.setColor(ExGraph.STANDARD_TEMP_NODE_COLOR);
			g2d.fillOval(node.getXpos() - node.getRadius() / 2, node.getYpos()
					- node.getRadius() / 2, node.getRadius(), node.getRadius());
			break;
		}
		case 2: // MOUSE OVER NODE DRAWING
		{
			myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f);
			g2d.setComposite(myAlpha);

			tempRadius = node.getRadius()
					+ ExGraph.STANDARD_MOUSEOVER_RADIUS_ADD;

			g2d.setColor(ExGraph.STANDARD_MOUSEOVER_NODE_COLOR);
			g2d.fillOval(node.getXpos() - tempRadius / 2, node.getYpos()
					- tempRadius / 2, tempRadius, tempRadius);
			break;
		}
		case 3: // ACTIVE NODE
		{
			myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f);
			g2d.setComposite(myAlpha);

			tempRadius = node.getRadius()
					+ ExGraph.STANDARD_MOUSEOVER_RADIUS_ADD + 4;

			g2d.setColor(ExGraph.STANDARD_ACTIVE_NODE_COLOR);
			g2d.fillOval(node.getXpos() - tempRadius / 2, node.getYpos()
					- tempRadius / 2, tempRadius, tempRadius);
			break;
		}

		default:
			return true;
		}

		return true;
	}

	public boolean drawEdge(ExEdge edge, Graphics2D g2d, int mode)
	{

		switch (mode)
		{

		case 0: // STANDARD EDGE

			String s = edge.getLabel();
			FontRenderContext frc = g2d.getFontRenderContext();
			Font f = new Font(ExGraph.STANDARD_LABEL_FONT, Font.PLAIN,
					(int) (ExGraph.STANDARD_LABEL_FONT_SIZE / 1.5));

			g2d.setColor(ExGraph.STANDARD_LABEL_COLOR);

			float x1 = edge.getStartNode().getXpos();
			float y1 = edge.getStartNode().getYpos();
			float x2 = edge.getEndNode().getXpos();
			float y2 = edge.getEndNode().getYpos();
			float w = (x2 - x1);
			float h = (y2 - y1);

			TextLayout tl = new TextLayout(s, f, frc);

			g2d.setColor(ExGraph.STANDARD_LABEL_COLOR);
			g2d.setStroke(new BasicStroke(ExGraph.STANDARD_EDGE_SIZE));

			// /herkmmliches label
			x1 += h / 20f;
			tl.draw(g2d, (float) (x1 + w / 2 - tl.getBounds().getCenterX()),
					(float) ((y1 + h / 2) + h / 20f));
			x1 -= h / 20f;

			// gerade linie
			g2d.setColor(ExGraph.STANDARD_EDGE_COLOR);
			g2d.drawLine(edge.getStartNode().getXpos(), edge.getStartNode()
					.getYpos(), edge.getEndNode().getXpos(), edge.getEndNode()
					.getYpos());

			// /////////pfeil

			float arrow = 7f;
			GeneralPath gp = new GeneralPath();

			gp.moveTo(x2, y2);
			gp.lineTo(x2 - arrow, y2 - (arrow / 2));
			gp.lineTo(x2 - arrow, y2 + (arrow / 2));
			gp.lineTo(x2, y2);

			AffineTransform t = new AffineTransform();
			t.setToTranslation(x2, y2);
			t.scale(2, 2);

			double angle = Math.atan(h / w);

			if (w < 0)
				angle = Math.toRadians(Math.toDegrees(angle) + 180);

			t.rotate(angle);
			t.translate(-x2 - (edge.getEndNode().getRadius() / 4), -y2);
			g2d.fill(t.createTransformedShape(gp));

			break;

		case 1: // TEMP EDGE DRAWING

			g2d.setColor(ExGraph.STANDARD_TEMP_EDGE_COLOR);
			g2d.setStroke(new BasicStroke(ExGraph.STANDARD_EDGE_SIZE));
			g2d.drawLine(edge.getStartNode().getXpos(), edge.getStartNode()
					.getYpos(), edge.getEndNode().getXpos(), edge.getEndNode()
					.getYpos());

			g2d.setColor(ExGraph.STANDARD_TEMP_LABEL_COLOR);

			x1 = (float) edge.getStartNode().getXpos();
			y1 = (float) edge.getStartNode().getYpos();
			x2 = (float) edge.getEndNode().getXpos();
			y2 = (float) edge.getEndNode().getYpos();

			w = (float) (x2 - x1);
			h = (float) (y2 - y1);
			Float l = (float) Math.abs(Math.sqrt((w * w + h * h)));

			g2d.setColor(ExGraph.STANDARD_TEMP_EDGE_COLOR);
			g2d.setStroke(new BasicStroke(ExGraph.STANDARD_EDGE_SIZE));

			// ///////pfeil

			arrow = 7f;
			gp = new GeneralPath();

			gp.moveTo(x2, y2);
			gp.lineTo(x2 - arrow, y2 - (arrow / 2));
			gp.lineTo(x2 - arrow, y2 + (arrow / 2));
			gp.lineTo(x2, y2);

			t = new AffineTransform();
			t.setToTranslation(x2, y2);
			t.scale(2, 2);

			angle = Math.atan(h / w);

			if (w < 0)
				angle = Math.toRadians(Math.toDegrees(angle) + 180);

			t.rotate(angle);
			t.translate(-x2, -y2);
			g2d.fill(t.createTransformedShape(gp));

			// ende pfeil
			break;

		case 2: // MOUSE OVER DRAWING

			g2d.setColor(ExGraph.STANDARD_MOUSEOVER_EDGE_COLOR);
			g2d.setStroke(new BasicStroke(ExGraph.STANDARD_EDGE_SIZE));
			g2d.drawLine(edge.getStartNode().getXpos(), edge.getStartNode()
					.getYpos(), edge.getEndNode().getXpos(), edge.getEndNode()
					.getYpos());

			g2d.setColor(ExGraph.STANDARD_TEMP_LABEL_COLOR);

			x1 = (float) edge.getStartNode().getXpos();
			y1 = (float) edge.getStartNode().getYpos();
			x2 = (float) edge.getEndNode().getXpos();
			y2 = (float) edge.getEndNode().getYpos();

			w = (float) (x2 - x1);
			h = (float) (y2 - y1);
			l = (float) Math.abs(Math.sqrt((w * w + h * h)));

			g2d.setColor(ExGraph.STANDARD_MOUSEOVER_EDGE_COLOR);
			g2d.setStroke(new BasicStroke(ExGraph.STANDARD_EDGE_SIZE));

			// /////////pfeil

			arrow = 7f;
			gp = new GeneralPath();

			gp.moveTo(x2, y2);
			gp.lineTo(x2 - arrow, y2 - (arrow / 2));
			gp.lineTo(x2 - arrow, y2 + (arrow / 2));
			gp.lineTo(x2, y2);

			t = new AffineTransform();
			t.setToTranslation(x2, y2);
			t.scale(2, 2);

			angle = Math.atan(h / w);

			if (w < 0)
				angle = Math.toRadians(Math.toDegrees(angle) + 180);

			t.rotate(angle);
			t.translate(-x2 - (edge.getEndNode().getRadius() / 4), -y2);
			g2d.fill(t.createTransformedShape(gp));
			break;
		case 3: // ACTIVE EDGE DRAWING

			g2d.setColor(ExGraph.STANDARD_ACTIVE_EDGE_COLOR);
			g2d.setStroke(new BasicStroke(ExGraph.STANDARD_EDGE_SIZE));
			g2d.drawLine(edge.getStartNode().getXpos(), edge.getStartNode()
					.getYpos(), edge.getEndNode().getXpos(), edge.getEndNode()
					.getYpos());

			g2d.setColor(ExGraph.STANDARD_TEMP_LABEL_COLOR);

			x1 = (float) edge.getStartNode().getXpos();
			y1 = (float) edge.getStartNode().getYpos();
			x2 = (float) edge.getEndNode().getXpos();
			y2 = (float) edge.getEndNode().getYpos();

			w = (float) (x2 - x1);
			h = (float) (y2 - y1);
			l = (float) Math.abs(Math.sqrt((w * w + h * h)));

			g2d.setColor(ExGraph.STANDARD_ACTIVE_EDGE_COLOR);
			g2d.setStroke(new BasicStroke(ExGraph.STANDARD_EDGE_SIZE));

			// tl.draw(g2d, (float) (edge.getStartNode().getXpos() +
			// (edge.getEndNode().getXpos()-edge.getStartNode().getXpos()) / 2 -
			// tl.getBounds().getCenterX()),
			// (float) (edge.getStartNode().getYpos() +
			// (edge.getEndNode().getYpos()-edge.getStartNode().getYpos()) /
			// 2));

			// gerade linie
			// g2d.drawLine(edge.getStartNode().getXpos(),
			// edge.getStartNode().getYpos(),
			// edge.getEndNode().getXpos(), edge.getEndNode().getYpos());

			// /////////pfeil

			arrow = 7f;
			gp = new GeneralPath();

			gp.moveTo(x2, y2);
			gp.lineTo(x2 - arrow, y2 - (arrow / 2));
			gp.lineTo(x2 - arrow, y2 + (arrow / 2));
			gp.lineTo(x2, y2);

			t = new AffineTransform();
			t.setToTranslation(x2, y2);
			t.scale(2, 2);

			angle = Math.atan(h / w);

			if (w < 0)
				angle = Math.toRadians(Math.toDegrees(angle) + 180);

			t.rotate(angle);
			t.translate(-x2, -y2);
			g2d.fill(t.createTransformedShape(gp));
			break;
		}

		return true;

	}

	public boolean drawLabel(ExEdge edge, Graphics2D g2d, int mode)
	{

		switch (mode)
		{

		case 0: // STANDARD LABEL DRAWING

			g2d.setColor(ExGraph.STANDARD_LABEL_COLOR);
			AlphaComposite myAlpha = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, ExGraph.STANDARD_LABEL_ALPHA);

			g2d.setComposite(myAlpha);

			Float x1 = (float) edge.getStartNode().getXpos();
			Float y1 = (float) edge.getStartNode().getYpos();
			Float x2 = (float) edge.getEndNode().getXpos();
			Float y2 = (float) edge.getEndNode().getYpos();

			if (ExGraph.STANDARD_LABEL_FONT_SIZE > 10
					&& ExGraph.DRAW_PERFORMANCE_MODE == false)
			{
				GradientPaint gradient = new GradientPaint(x1, y1, Color.gray,
						x2, y2, Color.green, true);
				g2d.setPaint(gradient);
				g2d.fill(edge.getShape());
			} else
			{
				g2d.setColor(ExGraph.STANDARD_TEMP_NODE_COLOR);
				g2d.fill(edge.getLodshape());
			}
			break;

		case 1: // TEMP LABEL DRAWING

			// setting alpha
			g2d.setColor(ExGraph.STANDARD_LABEL_COLOR);
			myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);

			// g2d.setColor(ExGraph.STANDARD_TEMP_LABEL_COLOR);

			x1 = (float) edge.getStartNode().getXpos();
			y1 = (float) edge.getStartNode().getYpos();
			x2 = (float) edge.getEndNode().getXpos();
			y2 = (float) edge.getEndNode().getYpos();

			if (ExGraph.STANDARD_LABEL_FONT_SIZE > 10
					&& ExGraph.DRAW_PERFORMANCE_MODE == false)
			{
				GradientPaint gradient = new GradientPaint(x1, y1, Color.black,
						x2, y2, Color.green, true);
				g2d.setPaint(gradient);
				g2d.fill(edge.getShape());
			} else
			{
				g2d.setColor(ExGraph.STANDARD_TEMP_NODE_COLOR);
				g2d.fill(edge.getLodshape());
			}
			break;

		case 2: // MOUSEOVER LABEL DRAWING

			x1 = (float) edge.getStartNode().getXpos();
			y1 = (float) edge.getStartNode().getYpos();
			x2 = (float) edge.getEndNode().getXpos();
			y2 = (float) edge.getEndNode().getYpos();

			if (ExGraph.STANDARD_LABEL_FONT_SIZE > 10
					&& ExGraph.DRAW_PERFORMANCE_MODE == false)
			{
				myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
						0.8f);
				g2d.setComposite(myAlpha);
				GradientPaint gradient = new GradientPaint(x1, y1,
						Color.yellow, x2, y2, Color.yellow, true);
				g2d.setPaint(gradient);
				g2d.fill(edge.getShape());
			} else
			{
				myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
						ExGraph.STANDARD_LABEL_ALPHA);
				g2d.setComposite(myAlpha);
				g2d.setColor(ExGraph.STANDARD_MOUSEOVER_LABEL_COLOR);
				g2d.fill(edge.getLodshape());
			}
			break;

		case 3: // ACTIVE LABEL DRAWING

			x1 = (float) edge.getStartNode().getXpos();
			y1 = (float) edge.getStartNode().getYpos();
			x2 = (float) edge.getEndNode().getXpos();
			y2 = (float) edge.getEndNode().getYpos();

			if (ExGraph.STANDARD_LABEL_FONT_SIZE > 10
					&& ExGraph.DRAW_PERFORMANCE_MODE == false)
			{
				myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
						0.8f);
				g2d.setComposite(myAlpha);
				GradientPaint gradient = new GradientPaint(x1, y1, Color.black,
						x2, y2, Color.black, true);
				g2d.setPaint(gradient);
				g2d.fill(edge.getShape());
			} else
			{
				myAlpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
						ExGraph.STANDARD_LABEL_ALPHA);
				g2d.setComposite(myAlpha);
				g2d.setColor(Color.black);
				g2d.fill(edge.getLodshape());
			}
			break;

		}

		return true;
	}

	// wie im paper (test) ******************************####################
	/*
	  Float advance=0.0f, nextAdvance=0.0f; int currentChar = 0;
	  // FontRenderContext frc = new FontRenderContext(null, true, true);
	  
	  GlyphVector glyphVector = f.createGlyphVector(frc, s); int length =
	  glyphVector.getNumGlyphs(); // anzahl buchstaben AffineTransform transf
									// =
	  new AffineTransform(); 
	  boolean finished = false; 
	  float angle =  (float)Math.atan2( h, w ); 
	  int i = 0; 
	  while ( i < s.length() && !finished ) {
	  advance = glyphVector.getGlyphMetrics(currentChar).getAdvance();//*0.50f; Point2D p = glyphVector.getGlyphPosition(currentChar+1);
	  // buchstabenposition Float px = (float)p.getX(); //x von buchstabe
		// Float
	  py = (float)p.getY();// y von buchstabe Shape glyph =
	  glyphVector.getGlyphOutline( currentChar ); 
	  
	  double zaehler =  Math.pow(px-w/2, 2); 
	  
	  double nenner= (-(w*w))/(4*h);
	  // System.out.println(nenner); 
	  py = (float)zaehler/(float)nenner;//(( (px-w)/2) * ( (px-w)/2) )/((w*w)/4*h); ystep=(float)py; py=y+py+h;
	  transf.setToTranslation(x, py); // transf.rotate( angle ); //
	  t.translate( -px-advance, -py ); // t.scale(1, scalef);
	  System.out.println("char:"+s.substring(i, i+1) +" :nextadv: " +  advance.intValue()+"\tx1: "+x.intValue()+"\ty1: "+y.intValue()+"\t __ x2: "+xend.intValue()+"\ty2: "+yend.intValue()+"\t __ px:	  "+px.intValue()+"\tpy: "+py.intValue()); ///
	  
	  g2d.fill(transf.createTransformedShape( glyph ));
	  g2d.fillOval(px.intValue()+x.intValue(), py.intValue(), 5, 5); //
	  System.out.println(x.intValue()+" "+py); currentChar++; i++; 
	  
	  if ( px >	  xend || py > yend-ystep) 
		  finished=true; 
	  }
	*/

	// // ******########## END: test ########################
}
