package exlabel.util;

import java.awt.Font;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.AffineTransform;
import java.awt.geom.FlatteningPathIterator;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;

public class TextStroke implements Stroke {
	private String text;
	private Font font;
	private float startoffset;
	private AffineTransform t = new AffineTransform();

	public TextStroke(String text, Font font) {
		this(text, font, 0f);
	}

	public TextStroke(String text, Font font, float startoffset) {
		this.text = text;
		this.font = font;	
		this.startoffset = startoffset;

	}

	public Shape createStrokedShape(Shape shape) 
	{
		FontRenderContext frc = new FontRenderContext(null, true, true);
		GlyphVector glyphVector = font.createGlyphVector(frc, text);
		GeneralPath result = new GeneralPath();
		PathIterator it = new FlatteningPathIterator(shape.getPathIterator(null), 1);
		float points[] = new float[6];
		float moveX = 0, moveY = 0;
		float lastX = 0, lastY = 0;
		float thisX = 0, thisY = 0;
		int type = 0;
		boolean first = true;
		float next = 0;
		int currentChar = 0;
		int length = glyphVector.getNumGlyphs();

		if (length == 0)
			return result;

		Float pathlength = measurePathLength(shape);		
		float factor = 1.0f; 
		float nextAdvance = 0;		
		double scalef = 1;		
		float r = 0f;
		
		while (currentChar < length && !it.isDone()) 
		{			
			type = it.currentSegment(points);

			switch (type) 
			{
			case PathIterator.SEG_MOVETO:
				
				moveX = lastX = points[0];
				moveY = lastY = points[1];
				result.moveTo(moveX, moveY);				
				nextAdvance = glyphVector.getGlyphMetrics(currentChar).getAdvance() * 0.5f; // x.xf abstand zw. buchstaben
								
				if (first) {
					first = false;
					next += nextAdvance + startoffset;
				} else
					next = nextAdvance;
				
				break;

			case PathIterator.SEG_LINETO:
				
				thisX = points[0];
				thisY = points[1];
				float dx = thisX - lastX;
				float dy = thisY - lastY;
				float distance = (float) Math.sqrt(dx * dx + dy * dy);
				r = 1 / distance;

				if (distance >= next) {
					float angle = (float) Math.atan2(dy, dx);

					while (currentChar < length && distance >= next) 
					{
						Shape glyph = glyphVector.getGlyphOutline(currentChar); 
						Point2D p = glyphVector.getGlyphPosition(currentChar);
						float px = (float) p.getX(); 
						float py = (float) p.getY(); 
						float x = lastX + next * dx * r;
						float y = lastY + next * dy * r;
						
						float advance = nextAdvance;
						nextAdvance = currentChar < length - 1						
								? (float) (glyphVector.getGlyphMetrics(currentChar).getBounds2D().getWidth())* 0.7f * (float) scalef : 
								10.5f;
								
						t.setToTranslation(x, y);
						t.scale(scalef, scalef);
						t.rotate(angle);
						t.translate(-px - advance, -py);
						
						// TODO animierten text

						result.append(t.createTransformedShape(glyph), false);						
						next += (advance + nextAdvance) * factor;
						currentChar++;
					
						currentChar = currentChar % length;
						nextAdvance = advance;					

						scalef *= 1 - ((1 / (pathlength)) * glyphVector.getGlyphMetrics(currentChar).getBounds2D().getWidth()*1.2);
						// System.out.println("char: "+currentChar+" scalef: "+scalef);
					}
				}
				next -= distance;				
				lastX = thisX;
				lastY = thisY;
				break;
			}
			it.next();

		}
		//System.out.println("===== textstroke finished =====");
		return result;

	}

	public float measurePathLength(Shape shape) {
		PathIterator it = new FlatteningPathIterator(shape
				.getPathIterator(null), 1);
		float points[] = new float[6];
		float moveX = 0, moveY = 0;
		float lastX = 0, lastY = 0;
		float thisX = 0, thisY = 0;
		int type = 0;
		float total = 0;

	
		while (!it.isDone()) {
			
			type = it.currentSegment(points);
			switch (type) {
			case PathIterator.SEG_MOVETO:
				//System.out.println("move");
				moveX = lastX = points[0];
				moveY = lastY = points[1];
				break;

			case PathIterator.SEG_CLOSE:
				//System.out.println("close");
				points[0] = moveX;
				points[1] = moveY;
			// Fall into....

			case PathIterator.SEG_LINETO:
				
				thisX = points[0];
				thisY = points[1];
			//	System.out.println("x: "+thisX+"   \ty:"+thisY);
				float dx = thisX - lastX;
				float dy = thisY - lastY;
				total += (float) Math.sqrt(dx * dx + dy * dy);
				lastX = thisX;
				lastY = thisY;
				break;
			}
			it.next();
		}

		return total;
	}

}
